# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import dashboard
from ..cog import variants as cogvariants
from ..models import table
from ..models import common


class Panel(cogbuilder.Builder[dashboard.Panel]):    
    """
    Dashboard panels are the basic visualization building blocks.
    """
    
    _internal: dashboard.Panel

    def __init__(self):
        self._internal = dashboard.Panel()        
        self._internal.type_val = "table"

    def build(self) -> dashboard.Panel:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: int) -> typing.Self:    
        """
        Unique identifier of the panel. Generated by Grafana when creating a new panel. It must be unique within a dashboard, but not globally.
        """
            
        self._internal.id_val = id_val
    
        return self
    
    def targets(self, targets: list[cogbuilder.Builder[cogvariants.Dataquery]]) -> typing.Self:    
        """
        Depends on the panel plugin. See the plugin documentation for details.
        """
            
        targets_resources = [r1.build() for r1 in targets]
        self._internal.targets = targets_resources
    
        return self
    
    def with_target(self, target: cogbuilder.Builder[cogvariants.Dataquery]) -> typing.Self:    
        """
        Depends on the panel plugin. See the plugin documentation for details.
        """
            
        if self._internal.targets is None:
            self._internal.targets = []
        
        target_resource = target.build()
        self._internal.targets.append(target_resource)
    
        return self
    
    def title(self, title: str) -> typing.Self:    
        """
        Panel title.
        """
            
        self._internal.title = title
    
        return self
    
    def description(self, description: str) -> typing.Self:    
        """
        Panel description.
        """
            
        self._internal.description = description
    
        return self
    
    def transparent(self, transparent: bool) -> typing.Self:    
        """
        Whether to display the panel without a background.
        """
            
        self._internal.transparent = transparent
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        The datasource used in all targets.
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def grid_pos(self, grid_pos: dashboard.GridPos) -> typing.Self:    
        """
        Grid position.
        """
            
        self._internal.grid_pos = grid_pos
    
        return self
    
    def height(self, h: int) -> typing.Self:    
        """
        Panel height. The height is the number of rows from the top edge of the panel.
        """
            
        if not h > 0:
            raise ValueError("h must be > 0")
        if self._internal.grid_pos is None:
            self._internal.grid_pos = dashboard.GridPos()
        assert isinstance(self._internal.grid_pos, dashboard.GridPos)
        self._internal.grid_pos.h = h
    
        return self
    
    def span(self, w: int) -> typing.Self:    
        """
        Panel width. The width is the number of columns from the left edge of the panel.
        """
            
        if not w > 0:
            raise ValueError("w must be > 0")
        if not w <= 24:
            raise ValueError("w must be <= 24")
        if self._internal.grid_pos is None:
            self._internal.grid_pos = dashboard.GridPos()
        assert isinstance(self._internal.grid_pos, dashboard.GridPos)
        self._internal.grid_pos.w = w
    
        return self
    
    def links(self, links: list[cogbuilder.Builder[dashboard.DashboardLink]]) -> typing.Self:    
        """
        Panel links.
        """
            
        links_resources = [r1.build() for r1 in links]
        self._internal.links = links_resources
    
        return self
    
    def repeat(self, repeat: str) -> typing.Self:    
        """
        Name of template variable to repeat for.
        """
            
        self._internal.repeat = repeat
    
        return self
    
    def repeat_direction(self, repeat_direction: typing.Literal["h", "v"]) -> typing.Self:    
        """
        Direction to repeat in if 'repeat' is set.
        `h` for horizontal, `v` for vertical.
        """
            
        self._internal.repeat_direction = repeat_direction
    
        return self
    
    def max_per_row(self, max_per_row: float) -> typing.Self:    
        """
        Option for repeated panels that controls max items per row
        Only relevant for horizontally repeated panels
        """
            
        self._internal.max_per_row = max_per_row
    
        return self
    
    def max_data_points(self, max_data_points: float) -> typing.Self:    
        """
        The maximum number of data points that the panel queries are retrieving.
        """
            
        self._internal.max_data_points = max_data_points
    
        return self
    
    def transformations(self, transformations: list[dashboard.DataTransformerConfig]) -> typing.Self:    
        """
        List of transformations that are applied to the panel data before rendering.
        When there are multiple transformations, Grafana applies them in the order they are listed.
        Each transformation creates a result set that then passes on to the next transformation in the processing pipeline.
        """
            
        self._internal.transformations = transformations
    
        return self
    
    def with_transformation(self, transformation: dashboard.DataTransformerConfig) -> typing.Self:    
        """
        List of transformations that are applied to the panel data before rendering.
        When there are multiple transformations, Grafana applies them in the order they are listed.
        Each transformation creates a result set that then passes on to the next transformation in the processing pipeline.
        """
            
        if self._internal.transformations is None:
            self._internal.transformations = []
        
        self._internal.transformations.append(transformation)
    
        return self
    
    def interval(self, interval: str) -> typing.Self:    
        """
        The min time interval setting defines a lower limit for the $__interval and $__interval_ms variables.
        This value must be formatted as a number followed by a valid time
        identifier like: "40s", "3d", etc.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.interval = interval
    
        return self
    
    def time_from(self, time_from: str) -> typing.Self:    
        """
        Overrides the relative time range for individual panels,
        which causes them to be different than what is selected in
        the dashboard time picker in the top-right corner of the dashboard. You can use this to show metrics from different
        time periods or days on the same dashboard.
        The value is formatted as time operation like: `now-5m` (Last 5 minutes), `now/d` (the day so far),
        `now-5d/d`(Last 5 days), `now/w` (This week so far), `now-2y/y` (Last 2 years).
        Note: Panel time overrides have no effect when the dashboard’s time range is absolute.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.time_from = time_from
    
        return self
    
    def time_shift(self, time_shift: str) -> typing.Self:    
        """
        Overrides the time range for individual panels by shifting its start and end relative to the time picker.
        For example, you can shift the time range for the panel to be two hours earlier than the dashboard time picker setting `2h`.
        Note: Panel time overrides have no effect when the dashboard’s time range is absolute.
        See: https://grafana.com/docs/grafana/latest/panels-visualizations/query-transform-data/#query-options
        """
            
        self._internal.time_shift = time_shift
    
        return self
    
    def hide_time_override(self, hide_time_override: bool) -> typing.Self:    
        """
        Controls if the timeFrom or timeShift overrides are shown in the panel header
        """
            
        self._internal.hide_time_override = hide_time_override
    
        return self
    
    def library_panel(self, library_panel: dashboard.LibraryPanelRef) -> typing.Self:    
        """
        Dynamically load the panel
        """
            
        self._internal.library_panel = library_panel
    
        return self
    
    def display_name(self, display_name: str) -> typing.Self:    
        """
        The display value for this field.  This supports template variables blank is auto
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.display_name = display_name
    
        return self
    
    def unit(self, unit: str) -> typing.Self:    
        """
        Unit a field should use. The unit you select is applied to all fields except time.
        You can use the units ID availables in Grafana or a custom unit.
        Available units in Grafana: https://github.com/grafana/grafana/blob/main/packages/grafana-data/src/valueFormats/categories.ts
        As custom unit, you can use the following formats:
        `suffix:<suffix>` for custom unit that should go after value.
        `prefix:<prefix>` for custom unit that should go before value.
        `time:<format>` For custom date time formats type for example `time:YYYY-MM-DD`.
        `si:<base scale><unit characters>` for custom SI units. For example: `si: mF`. This one is a bit more advanced as you can specify both a unit and the source data scale. So if your source data is represented as milli (thousands of) something prefix the unit with that SI scale character.
        `count:<unit>` for a custom count unit.
        `currency:<unit>` for custom a currency unit.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.unit = unit
    
        return self
    
    def decimals(self, decimals: float) -> typing.Self:    
        """
        Specify the number of decimals Grafana includes in the rendered value.
        If you leave this field blank, Grafana automatically truncates the number of decimals based on the value.
        For example 1.1234 will display as 1.12 and 100.456 will display as 100.
        To display all decimals, set the unit to `String`.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.decimals = decimals
    
        return self
    
    def min(self, min_val: float) -> typing.Self:    
        """
        The minimum value used in percentage threshold calculations. Leave blank for auto calculation based on all series and fields.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.min_val = min_val
    
        return self
    
    def max(self, max_val: float) -> typing.Self:    
        """
        The maximum value used in percentage threshold calculations. Leave blank for auto calculation based on all series and fields.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.max_val = max_val
    
        return self
    
    def mappings(self, mappings: list[dashboard.ValueMapping]) -> typing.Self:    
        """
        Convert input values into a display string
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.mappings = mappings
    
        return self
    
    def thresholds(self, thresholds: cogbuilder.Builder[dashboard.ThresholdsConfig]) -> typing.Self:    
        """
        Map numeric values to states
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        thresholds_resource = thresholds.build()
        self._internal.field_config.defaults.thresholds = thresholds_resource
    
        return self
    
    def color_scheme(self, color: cogbuilder.Builder[dashboard.FieldColor]) -> typing.Self:    
        """
        Panel color configuration
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        color_resource = color.build()
        self._internal.field_config.defaults.color = color_resource
    
        return self
    
    def data_links(self, links: list[cogbuilder.Builder[dashboard.DashboardLink]]) -> typing.Self:    
        """
        The behavior when clicking on a result
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        links_resources = [r1.build() for r1 in links]
        self._internal.field_config.defaults.links = links_resources
    
        return self
    
    def no_value(self, no_value: str) -> typing.Self:    
        """
        Alternative to empty string
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        self._internal.field_config.defaults.no_value = no_value
    
        return self
    
    def overrides(self, overrides: list[cogbuilder.Builder[dashboard.DashboardFieldConfigSourceOverrides]]) -> typing.Self:    
        """
        Overrides are the options applied to specific fields overriding the defaults.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        overrides_resources = [r1.build() for r1 in overrides]
        self._internal.field_config.overrides = overrides_resources
    
        return self
    
    def with_override(self, matcher: dashboard.MatcherConfig, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Overrides are the options applied to specific fields overriding the defaults.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=matcher,
            properties=properties,
        ))
    
        return self
    
    def override_by_name(self, name: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Adds override rules for a specific field, referred to by its name.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byName",
            options=name,
        ),
            properties=properties,
        ))
    
        return self
    
    def override_by_regexp(self, regexp: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Adds override rules for the fields whose name match the given regexp.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byRegexp",
            options=regexp,
        ),
            properties=properties,
        ))
    
        return self
    
    def override_by_field_type(self, field_type: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        """
        Adds override rules for all the fields of the given type.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byType",
            options=field_type,
        ),
            properties=properties,
        ))
    
        return self
    
    def override_by_query(self, query_ref_id: str, properties: list[dashboard.DynamicConfigValue]) -> typing.Self:    
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.overrides is None:
            self._internal.field_config.overrides = []
        
        self._internal.field_config.overrides.append(dashboard.DashboardFieldConfigSourceOverrides(
            matcher=dashboard.MatcherConfig(
            id_val="byFrameRefID",
            options=query_ref_id,
        ),
            properties=properties,
        ))
    
        return self
    
    def frame_index(self, frame_index: float) -> typing.Self:    
        """
        Represents the index of the selected frame
        """
            
        if self._internal.options is None:
            self._internal.options = table.Options()
        assert isinstance(self._internal.options, table.Options)
        self._internal.options.frame_index = frame_index
    
        return self
    
    def show_header(self, show_header: bool) -> typing.Self:    
        """
        Controls whether the panel should show the header
        """
            
        if self._internal.options is None:
            self._internal.options = table.Options()
        assert isinstance(self._internal.options, table.Options)
        self._internal.options.show_header = show_header
    
        return self
    
    def show_type_icons(self, show_type_icons: bool) -> typing.Self:    
        """
        Controls whether the header should show icons for the column types
        """
            
        if self._internal.options is None:
            self._internal.options = table.Options()
        assert isinstance(self._internal.options, table.Options)
        self._internal.options.show_type_icons = show_type_icons
    
        return self
    
    def sort_by(self, sort_by: list[cogbuilder.Builder[common.TableSortByFieldState]]) -> typing.Self:    
        """
        Used to control row sorting
        """
            
        if self._internal.options is None:
            self._internal.options = table.Options()
        assert isinstance(self._internal.options, table.Options)
        sort_by_resources = [r1.build() for r1 in sort_by]
        self._internal.options.sort_by = sort_by_resources
    
        return self
    
    def footer(self, footer: cogbuilder.Builder[common.TableFooterOptions]) -> typing.Self:    
        """
        Controls footer options
        """
            
        if self._internal.options is None:
            self._internal.options = table.Options()
        assert isinstance(self._internal.options, table.Options)
        footer_resource = footer.build()
        self._internal.options.footer = footer_resource
    
        return self
    
    def cell_height(self, cell_height: common.TableCellHeight) -> typing.Self:    
        """
        Controls the height of the rows
        """
            
        if self._internal.options is None:
            self._internal.options = table.Options()
        assert isinstance(self._internal.options, table.Options)
        self._internal.options.cell_height = cell_height
    
        return self
    
    def width(self, width: float) -> typing.Self:    
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        if self._internal.field_config.defaults.custom is None:
            self._internal.field_config.defaults.custom = table.FieldConfig()
        assert isinstance(self._internal.field_config.defaults.custom, table.FieldConfig)
        self._internal.field_config.defaults.custom.width = width
    
        return self
    
    def min_width(self, min_width: float) -> typing.Self:    
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        if self._internal.field_config.defaults.custom is None:
            self._internal.field_config.defaults.custom = table.FieldConfig()
        assert isinstance(self._internal.field_config.defaults.custom, table.FieldConfig)
        self._internal.field_config.defaults.custom.min_width = min_width
    
        return self
    
    def align(self, align: common.FieldTextAlignment) -> typing.Self:    
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        if self._internal.field_config.defaults.custom is None:
            self._internal.field_config.defaults.custom = table.FieldConfig()
        assert isinstance(self._internal.field_config.defaults.custom, table.FieldConfig)
        self._internal.field_config.defaults.custom.align = align
    
        return self
    
    def display_mode(self, display_mode: common.TableCellDisplayMode) -> typing.Self:    
        """
        This field is deprecated in favor of using cellOptions
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        if self._internal.field_config.defaults.custom is None:
            self._internal.field_config.defaults.custom = table.FieldConfig()
        assert isinstance(self._internal.field_config.defaults.custom, table.FieldConfig)
        self._internal.field_config.defaults.custom.display_mode = display_mode
    
        return self
    
    def cell_options(self, cell_options: common.TableCellOptions) -> typing.Self:    
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        if self._internal.field_config.defaults.custom is None:
            self._internal.field_config.defaults.custom = table.FieldConfig()
        assert isinstance(self._internal.field_config.defaults.custom, table.FieldConfig)
        self._internal.field_config.defaults.custom.cell_options = cell_options
    
        return self
    
    def hidden(self, hidden: bool) -> typing.Self:    
        """
        ?? default is missing or false ??
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        if self._internal.field_config.defaults.custom is None:
            self._internal.field_config.defaults.custom = table.FieldConfig()
        assert isinstance(self._internal.field_config.defaults.custom, table.FieldConfig)
        self._internal.field_config.defaults.custom.hidden = hidden
    
        return self
    
    def inspect(self, inspect: bool) -> typing.Self:    
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        if self._internal.field_config.defaults.custom is None:
            self._internal.field_config.defaults.custom = table.FieldConfig()
        assert isinstance(self._internal.field_config.defaults.custom, table.FieldConfig)
        self._internal.field_config.defaults.custom.inspect = inspect
    
        return self
    
    def filterable(self, filterable: bool) -> typing.Self:    
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        if self._internal.field_config.defaults.custom is None:
            self._internal.field_config.defaults.custom = table.FieldConfig()
        assert isinstance(self._internal.field_config.defaults.custom, table.FieldConfig)
        self._internal.field_config.defaults.custom.filterable = filterable
    
        return self
    
    def hide_header(self, hide_header: bool) -> typing.Self:    
        """
        Hides any header for a column, useful for columns that show some static content or buttons.
        """
            
        if self._internal.field_config is None:
            self._internal.field_config = dashboard.FieldConfigSource()
        assert isinstance(self._internal.field_config, dashboard.FieldConfigSource)
        if self._internal.field_config.defaults is None:
            self._internal.field_config.defaults = dashboard.FieldConfig()
        assert isinstance(self._internal.field_config.defaults, dashboard.FieldConfig)
        if self._internal.field_config.defaults.custom is None:
            self._internal.field_config.defaults.custom = table.FieldConfig()
        assert isinstance(self._internal.field_config.defaults.custom, table.FieldConfig)
        self._internal.field_config.defaults.custom.hide_header = hide_header
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

from ..models import common
import typing
from ..cog import runtime as cogruntime


class Options:
    # Set the height of the rows
    row_height: float
    # Show values on the columns
    show_value: common.VisibilityMode
    legend: common.VizLegendOptions
    tooltip: common.VizTooltipOptions
    timezone: typing.Optional[list[common.TimeZone]]
    # Controls the column width
    col_width: typing.Optional[float]

    def __init__(self, row_height: float = 0.9, show_value: typing.Optional[common.VisibilityMode] = None, legend: typing.Optional[common.VizLegendOptions] = None, tooltip: typing.Optional[common.VizTooltipOptions] = None, timezone: typing.Optional[list[common.TimeZone]] = None, col_width: typing.Optional[float] = 0.9):
        self.row_height = row_height
        self.show_value = show_value if show_value is not None else common.VisibilityMode.AUTO
        self.legend = legend if legend is not None else common.VizLegendOptions()
        self.tooltip = tooltip if tooltip is not None else common.VizTooltipOptions()
        self.timezone = timezone
        self.col_width = col_width

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "rowHeight": self.row_height,
            "showValue": self.show_value,
            "legend": self.legend,
            "tooltip": self.tooltip,
        }
        if self.timezone is not None:
            payload["timezone"] = self.timezone
        if self.col_width is not None:
            payload["colWidth"] = self.col_width
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "rowHeight" in data:
            args["row_height"] = data["rowHeight"]
        if "showValue" in data:
            args["show_value"] = data["showValue"]
        if "legend" in data:
            args["legend"] = common.VizLegendOptions.from_json(data["legend"])
        if "tooltip" in data:
            args["tooltip"] = common.VizTooltipOptions.from_json(data["tooltip"])
        if "timezone" in data:
            args["timezone"] = [item for item in data["timezone"]]
        if "colWidth" in data:
            args["col_width"] = data["colWidth"]        

        return cls(**args)


class FieldConfig:
    line_width: typing.Optional[int]
    hide_from: typing.Optional[common.HideSeriesConfig]
    fill_opacity: typing.Optional[int]

    def __init__(self, line_width: typing.Optional[int] = 1, hide_from: typing.Optional[common.HideSeriesConfig] = None, fill_opacity: typing.Optional[int] = 70):
        self.line_width = line_width
        self.hide_from = hide_from
        self.fill_opacity = fill_opacity

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.line_width is not None:
            payload["lineWidth"] = self.line_width
        if self.hide_from is not None:
            payload["hideFrom"] = self.hide_from
        if self.fill_opacity is not None:
            payload["fillOpacity"] = self.fill_opacity
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "lineWidth" in data:
            args["line_width"] = data["lineWidth"]
        if "hideFrom" in data:
            args["hide_from"] = common.HideSeriesConfig.from_json(data["hideFrom"])
        if "fillOpacity" in data:
            args["fill_opacity"] = data["fillOpacity"]        

        return cls(**args)





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="status-history",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=FieldConfig.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import elasticsearch
from ..models import dashboard


class DateHistogram(cogbuilder.Builder[elasticsearch.DateHistogram]):
    _internal: elasticsearch.DateHistogram

    def __init__(self):
        self._internal = elasticsearch.DateHistogram()

    def build(self) -> elasticsearch.DateHistogram:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchDateHistogramSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    


class Histogram(cogbuilder.Builder[elasticsearch.Histogram]):
    _internal: elasticsearch.Histogram

    def __init__(self):
        self._internal = elasticsearch.Histogram()

    def build(self) -> elasticsearch.Histogram:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchHistogramSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    


class Terms(cogbuilder.Builder[elasticsearch.Terms]):
    _internal: elasticsearch.Terms

    def __init__(self):
        self._internal = elasticsearch.Terms()

    def build(self) -> elasticsearch.Terms:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchTermsSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    


class Filters(cogbuilder.Builder[elasticsearch.Filters]):
    _internal: elasticsearch.Filters

    def __init__(self):
        self._internal = elasticsearch.Filters()

    def build(self) -> elasticsearch.Filters:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchFiltersSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    


class Filter(cogbuilder.Builder[elasticsearch.Filter]):
    _internal: elasticsearch.Filter

    def __init__(self):
        self._internal = elasticsearch.Filter()

    def build(self) -> elasticsearch.Filter:
        """
        Builds the object.
        """
        return self._internal    
    
    def query(self, query: str) -> typing.Self:    
        self._internal.query = query
    
        return self
    
    def label(self, label: str) -> typing.Self:    
        self._internal.label = label
    
        return self
    


class GeoHashGrid(cogbuilder.Builder[elasticsearch.GeoHashGrid]):
    _internal: elasticsearch.GeoHashGrid

    def __init__(self):
        self._internal = elasticsearch.GeoHashGrid()

    def build(self) -> elasticsearch.GeoHashGrid:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchGeoHashGridSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    


class Nested(cogbuilder.Builder[elasticsearch.Nested]):
    _internal: elasticsearch.Nested

    def __init__(self):
        self._internal = elasticsearch.Nested()

    def build(self) -> elasticsearch.Nested:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: object) -> typing.Self:    
        self._internal.settings = settings
    
        return self
    


class Count(cogbuilder.Builder[elasticsearch.Count]):
    _internal: elasticsearch.Count

    def __init__(self):
        self._internal = elasticsearch.Count()

    def build(self) -> elasticsearch.Count:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class MovingAverage(cogbuilder.Builder[elasticsearch.MovingAverage]):    
    """
    #MovingAverage's settings are overridden in types.ts
    """
    
    _internal: elasticsearch.MovingAverage

    def __init__(self):
        self._internal = elasticsearch.MovingAverage()

    def build(self) -> elasticsearch.MovingAverage:
        """
        Builds the object.
        """
        return self._internal    
    
    def pipeline_agg(self, pipeline_agg: str) -> typing.Self:    
        self._internal.pipeline_agg = pipeline_agg
    
        return self
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: dict[str, object]) -> typing.Self:    
        self._internal.settings = settings
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class Derivative(cogbuilder.Builder[elasticsearch.Derivative]):
    _internal: elasticsearch.Derivative

    def __init__(self):
        self._internal = elasticsearch.Derivative()

    def build(self) -> elasticsearch.Derivative:
        """
        Builds the object.
        """
        return self._internal    
    
    def pipeline_agg(self, pipeline_agg: str) -> typing.Self:    
        self._internal.pipeline_agg = pipeline_agg
    
        return self
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchDerivativeSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class CumulativeSum(cogbuilder.Builder[elasticsearch.CumulativeSum]):
    _internal: elasticsearch.CumulativeSum

    def __init__(self):
        self._internal = elasticsearch.CumulativeSum()

    def build(self) -> elasticsearch.CumulativeSum:
        """
        Builds the object.
        """
        return self._internal    
    
    def pipeline_agg(self, pipeline_agg: str) -> typing.Self:    
        self._internal.pipeline_agg = pipeline_agg
    
        return self
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchCumulativeSumSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class BucketScript(cogbuilder.Builder[elasticsearch.BucketScript]):
    _internal: elasticsearch.BucketScript

    def __init__(self):
        self._internal = elasticsearch.BucketScript()

    def build(self) -> elasticsearch.BucketScript:
        """
        Builds the object.
        """
        return self._internal    
    
    def pipeline_variables(self, pipeline_variables: list[cogbuilder.Builder[elasticsearch.PipelineVariable]]) -> typing.Self:    
        pipeline_variables_resources = [r1.build() for r1 in pipeline_variables]
        self._internal.pipeline_variables = pipeline_variables_resources
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchBucketScriptSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class PipelineVariable(cogbuilder.Builder[elasticsearch.PipelineVariable]):
    _internal: elasticsearch.PipelineVariable

    def __init__(self):
        self._internal = elasticsearch.PipelineVariable()

    def build(self) -> elasticsearch.PipelineVariable:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        self._internal.name = name
    
        return self
    
    def pipeline_agg(self, pipeline_agg: str) -> typing.Self:    
        self._internal.pipeline_agg = pipeline_agg
    
        return self
    


class SerialDiff(cogbuilder.Builder[elasticsearch.SerialDiff]):
    _internal: elasticsearch.SerialDiff

    def __init__(self):
        self._internal = elasticsearch.SerialDiff()

    def build(self) -> elasticsearch.SerialDiff:
        """
        Builds the object.
        """
        return self._internal    
    
    def pipeline_agg(self, pipeline_agg: str) -> typing.Self:    
        self._internal.pipeline_agg = pipeline_agg
    
        return self
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchSerialDiffSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class RawData(cogbuilder.Builder[elasticsearch.RawData]):
    _internal: elasticsearch.RawData

    def __init__(self):
        self._internal = elasticsearch.RawData()

    def build(self) -> elasticsearch.RawData:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchRawDataSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class RawDocument(cogbuilder.Builder[elasticsearch.RawDocument]):
    _internal: elasticsearch.RawDocument

    def __init__(self):
        self._internal = elasticsearch.RawDocument()

    def build(self) -> elasticsearch.RawDocument:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchRawDocumentSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class UniqueCount(cogbuilder.Builder[elasticsearch.UniqueCount]):
    _internal: elasticsearch.UniqueCount

    def __init__(self):
        self._internal = elasticsearch.UniqueCount()

    def build(self) -> elasticsearch.UniqueCount:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchUniqueCountSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class Percentiles(cogbuilder.Builder[elasticsearch.Percentiles]):
    _internal: elasticsearch.Percentiles

    def __init__(self):
        self._internal = elasticsearch.Percentiles()

    def build(self) -> elasticsearch.Percentiles:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchPercentilesSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class ExtendedStats(cogbuilder.Builder[elasticsearch.ExtendedStats]):
    _internal: elasticsearch.ExtendedStats

    def __init__(self):
        self._internal = elasticsearch.ExtendedStats()

    def build(self) -> elasticsearch.ExtendedStats:
        """
        Builds the object.
        """
        return self._internal    
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchExtendedStatsSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def meta(self, meta: object) -> typing.Self:    
        self._internal.meta = meta
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class Min(cogbuilder.Builder[elasticsearch.Min]):
    _internal: elasticsearch.Min

    def __init__(self):
        self._internal = elasticsearch.Min()

    def build(self) -> elasticsearch.Min:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchMinSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class Max(cogbuilder.Builder[elasticsearch.Max]):
    _internal: elasticsearch.Max

    def __init__(self):
        self._internal = elasticsearch.Max()

    def build(self) -> elasticsearch.Max:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchMaxSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class Sum(cogbuilder.Builder[elasticsearch.Sum]):
    _internal: elasticsearch.Sum

    def __init__(self):
        self._internal = elasticsearch.Sum()

    def build(self) -> elasticsearch.Sum:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchSumSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class Average(cogbuilder.Builder[elasticsearch.Average]):
    _internal: elasticsearch.Average

    def __init__(self):
        self._internal = elasticsearch.Average()

    def build(self) -> elasticsearch.Average:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchAverageSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class MovingFunction(cogbuilder.Builder[elasticsearch.MovingFunction]):
    _internal: elasticsearch.MovingFunction

    def __init__(self):
        self._internal = elasticsearch.MovingFunction()

    def build(self) -> elasticsearch.MovingFunction:
        """
        Builds the object.
        """
        return self._internal    
    
    def pipeline_agg(self, pipeline_agg: str) -> typing.Self:    
        self._internal.pipeline_agg = pipeline_agg
    
        return self
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchMovingFunctionSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class Logs(cogbuilder.Builder[elasticsearch.Logs]):
    _internal: elasticsearch.Logs

    def __init__(self):
        self._internal = elasticsearch.Logs()

    def build(self) -> elasticsearch.Logs:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchLogsSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class Rate(cogbuilder.Builder[elasticsearch.Rate]):
    _internal: elasticsearch.Rate

    def __init__(self):
        self._internal = elasticsearch.Rate()

    def build(self) -> elasticsearch.Rate:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchRateSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class TopMetrics(cogbuilder.Builder[elasticsearch.TopMetrics]):
    _internal: elasticsearch.TopMetrics

    def __init__(self):
        self._internal = elasticsearch.TopMetrics()

    def build(self) -> elasticsearch.TopMetrics:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchTopMetricsSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class BaseBucketAggregation(cogbuilder.Builder[elasticsearch.BaseBucketAggregation]):
    _internal: elasticsearch.BaseBucketAggregation

    def __init__(self):
        self._internal = elasticsearch.BaseBucketAggregation()

    def build(self) -> elasticsearch.BaseBucketAggregation:
        """
        Builds the object.
        """
        return self._internal    
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def type(self, type_val: elasticsearch.BucketAggregationType) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def settings(self, settings: object) -> typing.Self:    
        self._internal.settings = settings
    
        return self
    


class BucketAggregationWithField(cogbuilder.Builder[elasticsearch.BucketAggregationWithField]):
    _internal: elasticsearch.BucketAggregationWithField

    def __init__(self):
        self._internal = elasticsearch.BucketAggregationWithField()

    def build(self) -> elasticsearch.BucketAggregationWithField:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def type(self, type_val: elasticsearch.BucketAggregationType) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def settings(self, settings: object) -> typing.Self:    
        self._internal.settings = settings
    
        return self
    


class DateHistogramSettings(cogbuilder.Builder[elasticsearch.DateHistogramSettings]):
    _internal: elasticsearch.DateHistogramSettings

    def __init__(self):
        self._internal = elasticsearch.DateHistogramSettings()

    def build(self) -> elasticsearch.DateHistogramSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def interval(self, interval: str) -> typing.Self:    
        self._internal.interval = interval
    
        return self
    
    def min_doc_count(self, min_doc_count: str) -> typing.Self:    
        self._internal.min_doc_count = min_doc_count
    
        return self
    
    def trim_edges(self, trim_edges: str) -> typing.Self:    
        self._internal.trim_edges = trim_edges
    
        return self
    
    def offset(self, offset: str) -> typing.Self:    
        self._internal.offset = offset
    
        return self
    
    def time_zone(self, time_zone: str) -> typing.Self:    
        self._internal.time_zone = time_zone
    
        return self
    


class HistogramSettings(cogbuilder.Builder[elasticsearch.HistogramSettings]):
    _internal: elasticsearch.HistogramSettings

    def __init__(self):
        self._internal = elasticsearch.HistogramSettings()

    def build(self) -> elasticsearch.HistogramSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def interval(self, interval: str) -> typing.Self:    
        self._internal.interval = interval
    
        return self
    
    def min_doc_count(self, min_doc_count: str) -> typing.Self:    
        self._internal.min_doc_count = min_doc_count
    
        return self
    


class TermsSettings(cogbuilder.Builder[elasticsearch.TermsSettings]):
    _internal: elasticsearch.TermsSettings

    def __init__(self):
        self._internal = elasticsearch.TermsSettings()

    def build(self) -> elasticsearch.TermsSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def order(self, order: elasticsearch.TermsOrder) -> typing.Self:    
        self._internal.order = order
    
        return self
    
    def size(self, size: str) -> typing.Self:    
        self._internal.size = size
    
        return self
    
    def min_doc_count(self, min_doc_count: str) -> typing.Self:    
        self._internal.min_doc_count = min_doc_count
    
        return self
    
    def order_by(self, order_by: str) -> typing.Self:    
        self._internal.order_by = order_by
    
        return self
    
    def missing(self, missing: str) -> typing.Self:    
        self._internal.missing = missing
    
        return self
    


class FiltersSettings(cogbuilder.Builder[elasticsearch.FiltersSettings]):
    _internal: elasticsearch.FiltersSettings

    def __init__(self):
        self._internal = elasticsearch.FiltersSettings()

    def build(self) -> elasticsearch.FiltersSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def filters(self, filters: list[cogbuilder.Builder[elasticsearch.Filter]]) -> typing.Self:    
        filters_resources = [r1.build() for r1 in filters]
        self._internal.filters = filters_resources
    
        return self
    


class GeoHashGridSettings(cogbuilder.Builder[elasticsearch.GeoHashGridSettings]):
    _internal: elasticsearch.GeoHashGridSettings

    def __init__(self):
        self._internal = elasticsearch.GeoHashGridSettings()

    def build(self) -> elasticsearch.GeoHashGridSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def precision(self, precision: str) -> typing.Self:    
        self._internal.precision = precision
    
        return self
    


class BaseMetricAggregation(cogbuilder.Builder[elasticsearch.BaseMetricAggregation]):
    _internal: elasticsearch.BaseMetricAggregation

    def __init__(self):
        self._internal = elasticsearch.BaseMetricAggregation()

    def build(self) -> elasticsearch.BaseMetricAggregation:
        """
        Builds the object.
        """
        return self._internal    
    
    def type(self, type_val: elasticsearch.MetricAggregationType) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class MetricAggregationWithField(cogbuilder.Builder[elasticsearch.MetricAggregationWithField]):
    _internal: elasticsearch.MetricAggregationWithField

    def __init__(self):
        self._internal = elasticsearch.MetricAggregationWithField()

    def build(self) -> elasticsearch.MetricAggregationWithField:
        """
        Builds the object.
        """
        return self._internal    
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def type(self, type_val: elasticsearch.MetricAggregationType) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class MetricAggregationWithMissingSupport(cogbuilder.Builder[elasticsearch.MetricAggregationWithMissingSupport]):
    _internal: elasticsearch.MetricAggregationWithMissingSupport

    def __init__(self):
        self._internal = elasticsearch.MetricAggregationWithMissingSupport()

    def build(self) -> elasticsearch.MetricAggregationWithMissingSupport:
        """
        Builds the object.
        """
        return self._internal    
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchMetricAggregationWithMissingSupportSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def type(self, type_val: elasticsearch.MetricAggregationType) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class MetricAggregationWithInlineScript(cogbuilder.Builder[elasticsearch.MetricAggregationWithInlineScript]):
    _internal: elasticsearch.MetricAggregationWithInlineScript

    def __init__(self):
        self._internal = elasticsearch.MetricAggregationWithInlineScript()

    def build(self) -> elasticsearch.MetricAggregationWithInlineScript:
        """
        Builds the object.
        """
        return self._internal    
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchMetricAggregationWithInlineScriptSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def type(self, type_val: elasticsearch.MetricAggregationType) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class ExtendedStat(cogbuilder.Builder[elasticsearch.ExtendedStat]):
    _internal: elasticsearch.ExtendedStat

    def __init__(self):
        self._internal = elasticsearch.ExtendedStat()

    def build(self) -> elasticsearch.ExtendedStat:
        """
        Builds the object.
        """
        return self._internal    
    
    def label(self, label: str) -> typing.Self:    
        self._internal.label = label
    
        return self
    
    def value(self, value: elasticsearch.ExtendedStatMetaType) -> typing.Self:    
        self._internal.value = value
    
        return self
    


class BasePipelineMetricAggregation(cogbuilder.Builder[elasticsearch.BasePipelineMetricAggregation]):
    _internal: elasticsearch.BasePipelineMetricAggregation

    def __init__(self):
        self._internal = elasticsearch.BasePipelineMetricAggregation()

    def build(self) -> elasticsearch.BasePipelineMetricAggregation:
        """
        Builds the object.
        """
        return self._internal    
    
    def pipeline_agg(self, pipeline_agg: str) -> typing.Self:    
        self._internal.pipeline_agg = pipeline_agg
    
        return self
    
    def field(self, field: str) -> typing.Self:    
        self._internal.field = field
    
        return self
    
    def type(self, type_val: str) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class PipelineMetricAggregationWithMultipleBucketPaths(cogbuilder.Builder[elasticsearch.PipelineMetricAggregationWithMultipleBucketPaths]):
    _internal: elasticsearch.PipelineMetricAggregationWithMultipleBucketPaths

    def __init__(self):
        self._internal = elasticsearch.PipelineMetricAggregationWithMultipleBucketPaths()

    def build(self) -> elasticsearch.PipelineMetricAggregationWithMultipleBucketPaths:
        """
        Builds the object.
        """
        return self._internal    
    
    def pipeline_variables(self, pipeline_variables: list[cogbuilder.Builder[elasticsearch.PipelineVariable]]) -> typing.Self:    
        pipeline_variables_resources = [r1.build() for r1 in pipeline_variables]
        self._internal.pipeline_variables = pipeline_variables_resources
    
        return self
    
    def type(self, type_val: elasticsearch.MetricAggregationType) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def id(self, id_val: str) -> typing.Self:    
        self._internal.id_val = id_val
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        self._internal.hide = hide
    
        return self
    


class MovingAverageModelOption(cogbuilder.Builder[elasticsearch.MovingAverageModelOption]):
    _internal: elasticsearch.MovingAverageModelOption

    def __init__(self):
        self._internal = elasticsearch.MovingAverageModelOption()

    def build(self) -> elasticsearch.MovingAverageModelOption:
        """
        Builds the object.
        """
        return self._internal    
    
    def label(self, label: str) -> typing.Self:    
        self._internal.label = label
    
        return self
    
    def value(self, value: elasticsearch.MovingAverageModel) -> typing.Self:    
        self._internal.value = value
    
        return self
    


class BaseMovingAverageModelSettings(cogbuilder.Builder[elasticsearch.BaseMovingAverageModelSettings]):
    _internal: elasticsearch.BaseMovingAverageModelSettings

    def __init__(self):
        self._internal = elasticsearch.BaseMovingAverageModelSettings()

    def build(self) -> elasticsearch.BaseMovingAverageModelSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def model(self, model: elasticsearch.MovingAverageModel) -> typing.Self:    
        self._internal.model = model
    
        return self
    
    def window(self, window: str) -> typing.Self:    
        self._internal.window = window
    
        return self
    
    def predict(self, predict: str) -> typing.Self:    
        self._internal.predict = predict
    
        return self
    


class MovingAverageSimpleModelSettings(cogbuilder.Builder[elasticsearch.MovingAverageSimpleModelSettings]):
    _internal: elasticsearch.MovingAverageSimpleModelSettings

    def __init__(self):
        self._internal = elasticsearch.MovingAverageSimpleModelSettings()

    def build(self) -> elasticsearch.MovingAverageSimpleModelSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def window(self, window: str) -> typing.Self:    
        self._internal.window = window
    
        return self
    
    def predict(self, predict: str) -> typing.Self:    
        self._internal.predict = predict
    
        return self
    


class MovingAverageLinearModelSettings(cogbuilder.Builder[elasticsearch.MovingAverageLinearModelSettings]):
    _internal: elasticsearch.MovingAverageLinearModelSettings

    def __init__(self):
        self._internal = elasticsearch.MovingAverageLinearModelSettings()

    def build(self) -> elasticsearch.MovingAverageLinearModelSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def window(self, window: str) -> typing.Self:    
        self._internal.window = window
    
        return self
    
    def predict(self, predict: str) -> typing.Self:    
        self._internal.predict = predict
    
        return self
    


class MovingAverageEWMAModelSettings(cogbuilder.Builder[elasticsearch.MovingAverageEWMAModelSettings]):
    _internal: elasticsearch.MovingAverageEWMAModelSettings

    def __init__(self):
        self._internal = elasticsearch.MovingAverageEWMAModelSettings()

    def build(self) -> elasticsearch.MovingAverageEWMAModelSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchMovingAverageEWMAModelSettingsSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def window(self, window: str) -> typing.Self:    
        self._internal.window = window
    
        return self
    
    def minimize(self, minimize: bool) -> typing.Self:    
        self._internal.minimize = minimize
    
        return self
    
    def predict(self, predict: str) -> typing.Self:    
        self._internal.predict = predict
    
        return self
    


class MovingAverageHoltModelSettings(cogbuilder.Builder[elasticsearch.MovingAverageHoltModelSettings]):
    _internal: elasticsearch.MovingAverageHoltModelSettings

    def __init__(self):
        self._internal = elasticsearch.MovingAverageHoltModelSettings()

    def build(self) -> elasticsearch.MovingAverageHoltModelSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchMovingAverageHoltModelSettingsSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def window(self, window: str) -> typing.Self:    
        self._internal.window = window
    
        return self
    
    def minimize(self, minimize: bool) -> typing.Self:    
        self._internal.minimize = minimize
    
        return self
    
    def predict(self, predict: str) -> typing.Self:    
        self._internal.predict = predict
    
        return self
    


class MovingAverageHoltWintersModelSettings(cogbuilder.Builder[elasticsearch.MovingAverageHoltWintersModelSettings]):
    _internal: elasticsearch.MovingAverageHoltWintersModelSettings

    def __init__(self):
        self._internal = elasticsearch.MovingAverageHoltWintersModelSettings()

    def build(self) -> elasticsearch.MovingAverageHoltWintersModelSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def settings(self, settings: cogbuilder.Builder[elasticsearch.ElasticsearchMovingAverageHoltWintersModelSettingsSettings]) -> typing.Self:    
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def window(self, window: str) -> typing.Self:    
        self._internal.window = window
    
        return self
    
    def minimize(self, minimize: bool) -> typing.Self:    
        self._internal.minimize = minimize
    
        return self
    
    def predict(self, predict: str) -> typing.Self:    
        self._internal.predict = predict
    
        return self
    


class Dataquery(cogbuilder.Builder[elasticsearch.Dataquery]):
    _internal: elasticsearch.Dataquery

    def __init__(self):
        self._internal = elasticsearch.Dataquery()

    def build(self) -> elasticsearch.Dataquery:
        """
        Builds the object.
        """
        return self._internal    
    
    def alias(self, alias: str) -> typing.Self:    
        """
        Alias pattern
        """
            
        self._internal.alias = alias
    
        return self
    
    def query(self, query: str) -> typing.Self:    
        """
        Lucene query
        """
            
        self._internal.query = query
    
        return self
    
    def time_field(self, time_field: str) -> typing.Self:    
        """
        Name of time field
        """
            
        self._internal.time_field = time_field
    
        return self
    
    def bucket_aggs(self, bucket_aggs: list[cogbuilder.Builder[elasticsearch.BucketAggregation]]) -> typing.Self:    
        """
        List of bucket aggregations
        """
            
        bucket_aggs_resources = [r1.build() for r1 in bucket_aggs]
        self._internal.bucket_aggs = bucket_aggs_resources
    
        return self
    
    def metrics(self, metrics: list[cogbuilder.Builder[elasticsearch.MetricAggregation]]) -> typing.Self:    
        """
        List of metric aggregations
        """
            
        metrics_resources = [r1.build() for r1 in metrics]
        self._internal.metrics = metrics_resources
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        A unique identifier for the query within the list of targets.
        In server side expressions, the refId is used as a variable name to identify results.
        By default, the UI will assign A->Z; however setting meaningful names may be useful.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        Note this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        Specify the query flavor
        TODO make this required and give it a default
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        For mixed data sources the selected datasource is on the query level.
        For non mixed scenarios this is undefined.
        TODO find a better way to do this ^ that's friendly to schema
        TODO this shouldn't be unknown but DataSourceRef | null
        """
            
        self._internal.datasource = datasource
    
        return self
    


class ElasticsearchDateHistogramSettings(cogbuilder.Builder[elasticsearch.ElasticsearchDateHistogramSettings]):
    _internal: elasticsearch.ElasticsearchDateHistogramSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchDateHistogramSettings()

    def build(self) -> elasticsearch.ElasticsearchDateHistogramSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def interval(self, interval: str) -> typing.Self:    
        self._internal.interval = interval
    
        return self
    
    def min_doc_count(self, min_doc_count: str) -> typing.Self:    
        self._internal.min_doc_count = min_doc_count
    
        return self
    
    def trim_edges(self, trim_edges: str) -> typing.Self:    
        self._internal.trim_edges = trim_edges
    
        return self
    
    def offset(self, offset: str) -> typing.Self:    
        self._internal.offset = offset
    
        return self
    
    def time_zone(self, time_zone: str) -> typing.Self:    
        self._internal.time_zone = time_zone
    
        return self
    


class ElasticsearchHistogramSettings(cogbuilder.Builder[elasticsearch.ElasticsearchHistogramSettings]):
    _internal: elasticsearch.ElasticsearchHistogramSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchHistogramSettings()

    def build(self) -> elasticsearch.ElasticsearchHistogramSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def interval(self, interval: str) -> typing.Self:    
        self._internal.interval = interval
    
        return self
    
    def min_doc_count(self, min_doc_count: str) -> typing.Self:    
        self._internal.min_doc_count = min_doc_count
    
        return self
    


class ElasticsearchTermsSettings(cogbuilder.Builder[elasticsearch.ElasticsearchTermsSettings]):
    _internal: elasticsearch.ElasticsearchTermsSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchTermsSettings()

    def build(self) -> elasticsearch.ElasticsearchTermsSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def order(self, order: elasticsearch.TermsOrder) -> typing.Self:    
        self._internal.order = order
    
        return self
    
    def size(self, size: str) -> typing.Self:    
        self._internal.size = size
    
        return self
    
    def min_doc_count(self, min_doc_count: str) -> typing.Self:    
        self._internal.min_doc_count = min_doc_count
    
        return self
    
    def order_by(self, order_by: str) -> typing.Self:    
        self._internal.order_by = order_by
    
        return self
    
    def missing(self, missing: str) -> typing.Self:    
        self._internal.missing = missing
    
        return self
    


class ElasticsearchFiltersSettings(cogbuilder.Builder[elasticsearch.ElasticsearchFiltersSettings]):
    _internal: elasticsearch.ElasticsearchFiltersSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchFiltersSettings()

    def build(self) -> elasticsearch.ElasticsearchFiltersSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def filters(self, filters: list[cogbuilder.Builder[elasticsearch.Filter]]) -> typing.Self:    
        filters_resources = [r1.build() for r1 in filters]
        self._internal.filters = filters_resources
    
        return self
    


class ElasticsearchGeoHashGridSettings(cogbuilder.Builder[elasticsearch.ElasticsearchGeoHashGridSettings]):
    _internal: elasticsearch.ElasticsearchGeoHashGridSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchGeoHashGridSettings()

    def build(self) -> elasticsearch.ElasticsearchGeoHashGridSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def precision(self, precision: str) -> typing.Self:    
        self._internal.precision = precision
    
        return self
    


class ElasticsearchDerivativeSettings(cogbuilder.Builder[elasticsearch.ElasticsearchDerivativeSettings]):
    _internal: elasticsearch.ElasticsearchDerivativeSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchDerivativeSettings()

    def build(self) -> elasticsearch.ElasticsearchDerivativeSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def unit(self, unit: str) -> typing.Self:    
        self._internal.unit = unit
    
        return self
    


class ElasticsearchCumulativeSumSettings(cogbuilder.Builder[elasticsearch.ElasticsearchCumulativeSumSettings]):
    _internal: elasticsearch.ElasticsearchCumulativeSumSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchCumulativeSumSettings()

    def build(self) -> elasticsearch.ElasticsearchCumulativeSumSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def format(self, format_val: str) -> typing.Self:    
        self._internal.format_val = format_val
    
        return self
    


class ElasticsearchBucketScriptSettings(cogbuilder.Builder[elasticsearch.ElasticsearchBucketScriptSettings]):
    _internal: elasticsearch.ElasticsearchBucketScriptSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchBucketScriptSettings()

    def build(self) -> elasticsearch.ElasticsearchBucketScriptSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def script(self, script: cogbuilder.Builder[elasticsearch.InlineScript]) -> typing.Self:    
        script_resource = script.build()
        self._internal.script = script_resource
    
        return self
    


class ElasticsearchInlineScript(cogbuilder.Builder[elasticsearch.ElasticsearchInlineScript]):
    _internal: elasticsearch.ElasticsearchInlineScript

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchInlineScript()

    def build(self) -> elasticsearch.ElasticsearchInlineScript:
        """
        Builds the object.
        """
        return self._internal    
    
    def inline(self, inline: str) -> typing.Self:    
        self._internal.inline = inline
    
        return self
    


class ElasticsearchSerialDiffSettings(cogbuilder.Builder[elasticsearch.ElasticsearchSerialDiffSettings]):
    _internal: elasticsearch.ElasticsearchSerialDiffSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchSerialDiffSettings()

    def build(self) -> elasticsearch.ElasticsearchSerialDiffSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def lag(self, lag: str) -> typing.Self:    
        self._internal.lag = lag
    
        return self
    


class ElasticsearchRawDataSettings(cogbuilder.Builder[elasticsearch.ElasticsearchRawDataSettings]):
    _internal: elasticsearch.ElasticsearchRawDataSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchRawDataSettings()

    def build(self) -> elasticsearch.ElasticsearchRawDataSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def size(self, size: str) -> typing.Self:    
        self._internal.size = size
    
        return self
    


class ElasticsearchRawDocumentSettings(cogbuilder.Builder[elasticsearch.ElasticsearchRawDocumentSettings]):
    _internal: elasticsearch.ElasticsearchRawDocumentSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchRawDocumentSettings()

    def build(self) -> elasticsearch.ElasticsearchRawDocumentSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def size(self, size: str) -> typing.Self:    
        self._internal.size = size
    
        return self
    


class ElasticsearchUniqueCountSettings(cogbuilder.Builder[elasticsearch.ElasticsearchUniqueCountSettings]):
    _internal: elasticsearch.ElasticsearchUniqueCountSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchUniqueCountSettings()

    def build(self) -> elasticsearch.ElasticsearchUniqueCountSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def precision_threshold(self, precision_threshold: str) -> typing.Self:    
        self._internal.precision_threshold = precision_threshold
    
        return self
    
    def missing(self, missing: str) -> typing.Self:    
        self._internal.missing = missing
    
        return self
    


class ElasticsearchPercentilesSettings(cogbuilder.Builder[elasticsearch.ElasticsearchPercentilesSettings]):
    _internal: elasticsearch.ElasticsearchPercentilesSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchPercentilesSettings()

    def build(self) -> elasticsearch.ElasticsearchPercentilesSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def script(self, script: cogbuilder.Builder[elasticsearch.InlineScript]) -> typing.Self:    
        script_resource = script.build()
        self._internal.script = script_resource
    
        return self
    
    def missing(self, missing: str) -> typing.Self:    
        self._internal.missing = missing
    
        return self
    
    def percents(self, percents: list[str]) -> typing.Self:    
        self._internal.percents = percents
    
        return self
    


class ElasticsearchExtendedStatsSettings(cogbuilder.Builder[elasticsearch.ElasticsearchExtendedStatsSettings]):
    _internal: elasticsearch.ElasticsearchExtendedStatsSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchExtendedStatsSettings()

    def build(self) -> elasticsearch.ElasticsearchExtendedStatsSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def script(self, script: cogbuilder.Builder[elasticsearch.InlineScript]) -> typing.Self:    
        script_resource = script.build()
        self._internal.script = script_resource
    
        return self
    
    def missing(self, missing: str) -> typing.Self:    
        self._internal.missing = missing
    
        return self
    
    def sigma(self, sigma: str) -> typing.Self:    
        self._internal.sigma = sigma
    
        return self
    


class ElasticsearchMinSettings(cogbuilder.Builder[elasticsearch.ElasticsearchMinSettings]):
    _internal: elasticsearch.ElasticsearchMinSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchMinSettings()

    def build(self) -> elasticsearch.ElasticsearchMinSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def script(self, script: cogbuilder.Builder[elasticsearch.InlineScript]) -> typing.Self:    
        script_resource = script.build()
        self._internal.script = script_resource
    
        return self
    
    def missing(self, missing: str) -> typing.Self:    
        self._internal.missing = missing
    
        return self
    


class ElasticsearchMaxSettings(cogbuilder.Builder[elasticsearch.ElasticsearchMaxSettings]):
    _internal: elasticsearch.ElasticsearchMaxSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchMaxSettings()

    def build(self) -> elasticsearch.ElasticsearchMaxSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def script(self, script: cogbuilder.Builder[elasticsearch.InlineScript]) -> typing.Self:    
        script_resource = script.build()
        self._internal.script = script_resource
    
        return self
    
    def missing(self, missing: str) -> typing.Self:    
        self._internal.missing = missing
    
        return self
    


class ElasticsearchSumSettings(cogbuilder.Builder[elasticsearch.ElasticsearchSumSettings]):
    _internal: elasticsearch.ElasticsearchSumSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchSumSettings()

    def build(self) -> elasticsearch.ElasticsearchSumSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def script(self, script: cogbuilder.Builder[elasticsearch.InlineScript]) -> typing.Self:    
        script_resource = script.build()
        self._internal.script = script_resource
    
        return self
    
    def missing(self, missing: str) -> typing.Self:    
        self._internal.missing = missing
    
        return self
    


class ElasticsearchAverageSettings(cogbuilder.Builder[elasticsearch.ElasticsearchAverageSettings]):
    _internal: elasticsearch.ElasticsearchAverageSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchAverageSettings()

    def build(self) -> elasticsearch.ElasticsearchAverageSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def script(self, script: cogbuilder.Builder[elasticsearch.InlineScript]) -> typing.Self:    
        script_resource = script.build()
        self._internal.script = script_resource
    
        return self
    
    def missing(self, missing: str) -> typing.Self:    
        self._internal.missing = missing
    
        return self
    


class ElasticsearchMovingFunctionSettings(cogbuilder.Builder[elasticsearch.ElasticsearchMovingFunctionSettings]):
    _internal: elasticsearch.ElasticsearchMovingFunctionSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchMovingFunctionSettings()

    def build(self) -> elasticsearch.ElasticsearchMovingFunctionSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def window(self, window: str) -> typing.Self:    
        self._internal.window = window
    
        return self
    
    def script(self, script: cogbuilder.Builder[elasticsearch.InlineScript]) -> typing.Self:    
        script_resource = script.build()
        self._internal.script = script_resource
    
        return self
    
    def shift(self, shift: str) -> typing.Self:    
        self._internal.shift = shift
    
        return self
    


class ElasticsearchLogsSettings(cogbuilder.Builder[elasticsearch.ElasticsearchLogsSettings]):
    _internal: elasticsearch.ElasticsearchLogsSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchLogsSettings()

    def build(self) -> elasticsearch.ElasticsearchLogsSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def limit(self, limit: str) -> typing.Self:    
        self._internal.limit = limit
    
        return self
    


class ElasticsearchRateSettings(cogbuilder.Builder[elasticsearch.ElasticsearchRateSettings]):
    _internal: elasticsearch.ElasticsearchRateSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchRateSettings()

    def build(self) -> elasticsearch.ElasticsearchRateSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def unit(self, unit: str) -> typing.Self:    
        self._internal.unit = unit
    
        return self
    
    def mode(self, mode: str) -> typing.Self:    
        self._internal.mode = mode
    
        return self
    


class ElasticsearchTopMetricsSettings(cogbuilder.Builder[elasticsearch.ElasticsearchTopMetricsSettings]):
    _internal: elasticsearch.ElasticsearchTopMetricsSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchTopMetricsSettings()

    def build(self) -> elasticsearch.ElasticsearchTopMetricsSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def order(self, order: str) -> typing.Self:    
        self._internal.order = order
    
        return self
    
    def order_by(self, order_by: str) -> typing.Self:    
        self._internal.order_by = order_by
    
        return self
    
    def metrics(self, metrics: list[str]) -> typing.Self:    
        self._internal.metrics = metrics
    
        return self
    


class ElasticsearchMetricAggregationWithMissingSupportSettings(cogbuilder.Builder[elasticsearch.ElasticsearchMetricAggregationWithMissingSupportSettings]):
    _internal: elasticsearch.ElasticsearchMetricAggregationWithMissingSupportSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchMetricAggregationWithMissingSupportSettings()

    def build(self) -> elasticsearch.ElasticsearchMetricAggregationWithMissingSupportSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def missing(self, missing: str) -> typing.Self:    
        self._internal.missing = missing
    
        return self
    


class ElasticsearchMetricAggregationWithInlineScriptSettings(cogbuilder.Builder[elasticsearch.ElasticsearchMetricAggregationWithInlineScriptSettings]):
    _internal: elasticsearch.ElasticsearchMetricAggregationWithInlineScriptSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchMetricAggregationWithInlineScriptSettings()

    def build(self) -> elasticsearch.ElasticsearchMetricAggregationWithInlineScriptSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def script(self, script: cogbuilder.Builder[elasticsearch.InlineScript]) -> typing.Self:    
        script_resource = script.build()
        self._internal.script = script_resource
    
        return self
    


class ElasticsearchMovingAverageEWMAModelSettingsSettings(cogbuilder.Builder[elasticsearch.ElasticsearchMovingAverageEWMAModelSettingsSettings]):
    _internal: elasticsearch.ElasticsearchMovingAverageEWMAModelSettingsSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchMovingAverageEWMAModelSettingsSettings()

    def build(self) -> elasticsearch.ElasticsearchMovingAverageEWMAModelSettingsSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def alpha(self, alpha: str) -> typing.Self:    
        self._internal.alpha = alpha
    
        return self
    


class ElasticsearchMovingAverageHoltModelSettingsSettings(cogbuilder.Builder[elasticsearch.ElasticsearchMovingAverageHoltModelSettingsSettings]):
    _internal: elasticsearch.ElasticsearchMovingAverageHoltModelSettingsSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchMovingAverageHoltModelSettingsSettings()

    def build(self) -> elasticsearch.ElasticsearchMovingAverageHoltModelSettingsSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def alpha(self, alpha: str) -> typing.Self:    
        self._internal.alpha = alpha
    
        return self
    
    def beta(self, beta: str) -> typing.Self:    
        self._internal.beta = beta
    
        return self
    


class ElasticsearchMovingAverageHoltWintersModelSettingsSettings(cogbuilder.Builder[elasticsearch.ElasticsearchMovingAverageHoltWintersModelSettingsSettings]):
    _internal: elasticsearch.ElasticsearchMovingAverageHoltWintersModelSettingsSettings

    def __init__(self):
        self._internal = elasticsearch.ElasticsearchMovingAverageHoltWintersModelSettingsSettings()

    def build(self) -> elasticsearch.ElasticsearchMovingAverageHoltWintersModelSettingsSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def alpha(self, alpha: str) -> typing.Self:    
        self._internal.alpha = alpha
    
        return self
    
    def beta(self, beta: str) -> typing.Self:    
        self._internal.beta = beta
    
        return self
    
    def gamma(self, gamma: str) -> typing.Self:    
        self._internal.gamma = gamma
    
        return self
    
    def period(self, period: str) -> typing.Self:    
        self._internal.period = period
    
        return self
    
    def pad(self, pad: bool) -> typing.Self:    
        self._internal.pad = pad
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import bigquery
from ..models import dashboard


class Dataquery(cogbuilder.Builder[bigquery.Dataquery]):    
    """
    Manually converted from https://github.com/grafana/google-bigquery-datasource/blob/18680e42ba557791d109c7c540c2c3f2647592f0/src/types.ts#L75
    """
    
    _internal: bigquery.Dataquery

    def __init__(self):
        self._internal = bigquery.Dataquery()

    def build(self) -> bigquery.Dataquery:
        """
        Builds the object.
        """
        return self._internal    
    
    def dataset(self, dataset: str) -> typing.Self:    
        self._internal.dataset = dataset
    
        return self
    
    def table(self, table: str) -> typing.Self:    
        self._internal.table = table
    
        return self
    
    def project(self, project: str) -> typing.Self:    
        self._internal.project = project
    
        return self
    
    def format(self, format_val: bigquery.QueryFormat) -> typing.Self:    
        self._internal.format_val = format_val
    
        return self
    
    def raw_query(self, raw_query: bool) -> typing.Self:    
        self._internal.raw_query = raw_query
    
        return self
    
    def raw_sql(self, raw_sql: str) -> typing.Self:    
        self._internal.raw_sql = raw_sql
    
        return self
    
    def location(self, location: str) -> typing.Self:    
        self._internal.location = location
    
        return self
    
    def partitioned(self, partitioned: bool) -> typing.Self:    
        self._internal.partitioned = partitioned
    
        return self
    
    def partitioned_field(self, partitioned_field: str) -> typing.Self:    
        self._internal.partitioned_field = partitioned_field
    
        return self
    
    def convert_to_utc(self, convert_to_utc: bool) -> typing.Self:    
        self._internal.convert_to_utc = convert_to_utc
    
        return self
    
    def sharded(self, sharded: bool) -> typing.Self:    
        self._internal.sharded = sharded
    
        return self
    
    def query_priority(self, query_priority: bigquery.QueryPriority) -> typing.Self:    
        self._internal.query_priority = query_priority
    
        return self
    
    def time_shift(self, time_shift: str) -> typing.Self:    
        self._internal.time_shift = time_shift
    
        return self
    
    def editor_mode(self, editor_mode: bigquery.EditorMode) -> typing.Self:    
        self._internal.editor_mode = editor_mode
    
        return self
    
    def sql(self, sql: cogbuilder.Builder[bigquery.SQLExpression]) -> typing.Self:    
        sql_resource = sql.build()
        self._internal.sql = sql_resource
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        A unique identifier for the query within the list of targets.
        In server side expressions, the refId is used as a variable name to identify results.
        By default, the UI will assign A->Z; however setting meaningful names may be useful.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        Note this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        Specify the query flavor
        TODO make this required and give it a default
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        For mixed data sources the selected datasource is on the query level.
        For non mixed scenarios this is undefined.
        TODO find a better way to do this ^ that's friendly to schema
        TODO this shouldn't be unknown but DataSourceRef | null
        """
            
        self._internal.datasource = datasource
    
        return self
    


class SQLExpression(cogbuilder.Builder[bigquery.SQLExpression]):
    _internal: bigquery.SQLExpression

    def __init__(self):
        self._internal = bigquery.SQLExpression()

    def build(self) -> bigquery.SQLExpression:
        """
        Builds the object.
        """
        return self._internal    
    
    def columns(self, columns: list[cogbuilder.Builder[bigquery.QueryEditorFunctionExpression]]) -> typing.Self:    
        columns_resources = [r1.build() for r1 in columns]
        self._internal.columns = columns_resources
    
        return self
    
    def from_val(self, from_val: str) -> typing.Self:    
        self._internal.from_val = from_val
    
        return self
    
    def where_string(self, where_string: str) -> typing.Self:    
        """
        whereJsonTree?: _
        """
            
        self._internal.where_string = where_string
    
        return self
    
    def group_by(self, group_by: list[cogbuilder.Builder[bigquery.QueryEditorGroupByExpression]]) -> typing.Self:    
        group_by_resources = [r1.build() for r1 in group_by]
        self._internal.group_by = group_by_resources
    
        return self
    
    def order_by(self, order_by: cogbuilder.Builder[bigquery.QueryEditorPropertyExpression]) -> typing.Self:    
        order_by_resource = order_by.build()
        self._internal.order_by = order_by_resource
    
        return self
    
    def order_by_direction(self, order_by_direction: bigquery.OrderByDirection) -> typing.Self:    
        self._internal.order_by_direction = order_by_direction
    
        return self
    
    def limit(self, limit: int) -> typing.Self:    
        self._internal.limit = limit
    
        return self
    
    def offset(self, offset: int) -> typing.Self:    
        self._internal.offset = offset
    
        return self
    


class QueryEditorFunctionExpression(cogbuilder.Builder[bigquery.QueryEditorFunctionExpression]):
    _internal: bigquery.QueryEditorFunctionExpression

    def __init__(self):
        self._internal = bigquery.QueryEditorFunctionExpression()

    def build(self) -> bigquery.QueryEditorFunctionExpression:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        self._internal.name = name
    
        return self
    
    def parameters(self, parameters: list[cogbuilder.Builder[bigquery.QueryEditorFunctionParameterExpression]]) -> typing.Self:    
        parameters_resources = [r1.build() for r1 in parameters]
        self._internal.parameters = parameters_resources
    
        return self
    


class QueryEditorFunctionParameterExpression(cogbuilder.Builder[bigquery.QueryEditorFunctionParameterExpression]):
    _internal: bigquery.QueryEditorFunctionParameterExpression

    def __init__(self):
        self._internal = bigquery.QueryEditorFunctionParameterExpression()

    def build(self) -> bigquery.QueryEditorFunctionParameterExpression:
        """
        Builds the object.
        """
        return self._internal    
    
    def name(self, name: str) -> typing.Self:    
        self._internal.name = name
    
        return self
    


class QueryEditorGroupByExpression(cogbuilder.Builder[bigquery.QueryEditorGroupByExpression]):
    _internal: bigquery.QueryEditorGroupByExpression

    def __init__(self):
        self._internal = bigquery.QueryEditorGroupByExpression()

    def build(self) -> bigquery.QueryEditorGroupByExpression:
        """
        Builds the object.
        """
        return self._internal    
    
    def property(self, property_val: cogbuilder.Builder[bigquery.QueryEditorProperty]) -> typing.Self:    
        property_val_resource = property_val.build()
        self._internal.property_val = property_val_resource
    
        return self
    


class QueryEditorProperty(cogbuilder.Builder[bigquery.QueryEditorProperty]):
    _internal: bigquery.QueryEditorProperty

    def __init__(self):
        self._internal = bigquery.QueryEditorProperty()

    def build(self) -> bigquery.QueryEditorProperty:
        """
        Builds the object.
        """
        return self._internal    
    
    def type(self, type_val: bigquery.QueryEditorPropertyType) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    
    def name(self, name: str) -> typing.Self:    
        self._internal.name = name
    
        return self
    


class QueryEditorPropertyExpression(cogbuilder.Builder[bigquery.QueryEditorPropertyExpression]):
    _internal: bigquery.QueryEditorPropertyExpression

    def __init__(self):
        self._internal = bigquery.QueryEditorPropertyExpression()

    def build(self) -> bigquery.QueryEditorPropertyExpression:
        """
        Builds the object.
        """
        return self._internal    
    
    def property(self, property_val: cogbuilder.Builder[bigquery.QueryEditorProperty]) -> typing.Self:    
        property_val_resource = property_val.build()
        self._internal.property_val = property_val_resource
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import expr
from ..models import dashboard


class TypeMath(cogbuilder.Builder[expr.TypeMath]):
    _internal: expr.TypeMath

    def __init__(self):
        self._internal = expr.TypeMath()        
        self._internal.type_val = "math"

    def build(self) -> expr.TypeMath:
        """
        Builds the object.
        """
        return self._internal    
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        The datasource
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def expression(self, expression: str) -> typing.Self:    
        """
        General math expression
        """
            
        if not len(expression) >= 1:
            raise ValueError("len(expression) must be >= 1")
        self._internal.expression = expression
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        NOTE: this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def interval_ms(self, interval_ms: float) -> typing.Self:    
        """
        Interval is the suggested duration between time points in a time series query.
        NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
        from the interval required to fill a pixels in the visualization
        """
            
        self._internal.interval_ms = interval_ms
    
        return self
    
    def max_data_points(self, max_data_points: int) -> typing.Self:    
        """
        MaxDataPoints is the maximum number of data points that should be returned from a time series query.
        NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
        from the number of pixels visible in a visualization
        """
            
        self._internal.max_data_points = max_data_points
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        QueryType is an optional identifier for the type of query.
        It can be used to distinguish different types of queries.
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        RefID is the unique identifier of the query, set by the frontend call.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def result_assertions(self, result_assertions: cogbuilder.Builder[expr.ExprTypeMathResultAssertions]) -> typing.Self:    
        """
        Optionally define expected query result behavior
        """
            
        result_assertions_resource = result_assertions.build()
        self._internal.result_assertions = result_assertions_resource
    
        return self
    
    def time_range(self, time_range: cogbuilder.Builder[expr.ExprTypeMathTimeRange]) -> typing.Self:    
        """
        TimeRange represents the query range
        NOTE: unlike generic /ds/query, we can now send explicit time values in each query
        NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
        """
            
        time_range_resource = time_range.build()
        self._internal.time_range = time_range_resource
    
        return self
    


class TypeReduce(cogbuilder.Builder[expr.TypeReduce]):
    _internal: expr.TypeReduce

    def __init__(self):
        self._internal = expr.TypeReduce()        
        self._internal.type_val = "reduce"

    def build(self) -> expr.TypeReduce:
        """
        Builds the object.
        """
        return self._internal    
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        The datasource
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def expression(self, expression: str) -> typing.Self:    
        """
        Reference to single query result
        """
            
        if not len(expression) >= 1:
            raise ValueError("len(expression) must be >= 1")
        self._internal.expression = expression
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        NOTE: this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def interval_ms(self, interval_ms: float) -> typing.Self:    
        """
        Interval is the suggested duration between time points in a time series query.
        NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
        from the interval required to fill a pixels in the visualization
        """
            
        self._internal.interval_ms = interval_ms
    
        return self
    
    def max_data_points(self, max_data_points: int) -> typing.Self:    
        """
        MaxDataPoints is the maximum number of data points that should be returned from a time series query.
        NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
        from the number of pixels visible in a visualization
        """
            
        self._internal.max_data_points = max_data_points
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        QueryType is an optional identifier for the type of query.
        It can be used to distinguish different types of queries.
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def reducer(self, reducer: typing.Literal["sum", "mean", "min", "max", "count", "last", "median"]) -> typing.Self:    
        """
        The reducer
        Possible enum values:
         - `"sum"` 
         - `"mean"` 
         - `"min"` 
         - `"max"` 
         - `"count"` 
         - `"last"` 
         - `"median"` 
        """
            
        self._internal.reducer = reducer
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        RefID is the unique identifier of the query, set by the frontend call.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def result_assertions(self, result_assertions: cogbuilder.Builder[expr.ExprTypeReduceResultAssertions]) -> typing.Self:    
        """
        Optionally define expected query result behavior
        """
            
        result_assertions_resource = result_assertions.build()
        self._internal.result_assertions = result_assertions_resource
    
        return self
    
    def settings(self, settings: cogbuilder.Builder[expr.ExprTypeReduceSettings]) -> typing.Self:    
        """
        Reducer Options
        """
            
        settings_resource = settings.build()
        self._internal.settings = settings_resource
    
        return self
    
    def time_range(self, time_range: cogbuilder.Builder[expr.ExprTypeReduceTimeRange]) -> typing.Self:    
        """
        TimeRange represents the query range
        NOTE: unlike generic /ds/query, we can now send explicit time values in each query
        NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
        """
            
        time_range_resource = time_range.build()
        self._internal.time_range = time_range_resource
    
        return self
    


class TypeResample(cogbuilder.Builder[expr.TypeResample]):
    _internal: expr.TypeResample

    def __init__(self):
        self._internal = expr.TypeResample()        
        self._internal.type_val = "resample"

    def build(self) -> expr.TypeResample:
        """
        Builds the object.
        """
        return self._internal    
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        The datasource
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def downsampler(self, downsampler: typing.Literal["sum", "mean", "min", "max", "count", "last", "median"]) -> typing.Self:    
        """
        The downsample function
        Possible enum values:
         - `"sum"` 
         - `"mean"` 
         - `"min"` 
         - `"max"` 
         - `"count"` 
         - `"last"` 
         - `"median"` 
        """
            
        self._internal.downsampler = downsampler
    
        return self
    
    def expression(self, expression: str) -> typing.Self:    
        """
        The math expression
        """
            
        if not len(expression) >= 1:
            raise ValueError("len(expression) must be >= 1")
        self._internal.expression = expression
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        NOTE: this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def interval_ms(self, interval_ms: float) -> typing.Self:    
        """
        Interval is the suggested duration between time points in a time series query.
        NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
        from the interval required to fill a pixels in the visualization
        """
            
        self._internal.interval_ms = interval_ms
    
        return self
    
    def max_data_points(self, max_data_points: int) -> typing.Self:    
        """
        MaxDataPoints is the maximum number of data points that should be returned from a time series query.
        NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
        from the number of pixels visible in a visualization
        """
            
        self._internal.max_data_points = max_data_points
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        QueryType is an optional identifier for the type of query.
        It can be used to distinguish different types of queries.
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        RefID is the unique identifier of the query, set by the frontend call.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def result_assertions(self, result_assertions: cogbuilder.Builder[expr.ExprTypeResampleResultAssertions]) -> typing.Self:    
        """
        Optionally define expected query result behavior
        """
            
        result_assertions_resource = result_assertions.build()
        self._internal.result_assertions = result_assertions_resource
    
        return self
    
    def time_range(self, time_range: cogbuilder.Builder[expr.ExprTypeResampleTimeRange]) -> typing.Self:    
        """
        TimeRange represents the query range
        NOTE: unlike generic /ds/query, we can now send explicit time values in each query
        NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
        """
            
        time_range_resource = time_range.build()
        self._internal.time_range = time_range_resource
    
        return self
    
    def upsampler(self, upsampler: typing.Literal["pad", "backfilling", "fillna"]) -> typing.Self:    
        """
        The upsample function
        Possible enum values:
         - `"pad"` Use the last seen value
         - `"backfilling"` backfill
         - `"fillna"` Do not fill values (nill)
        """
            
        self._internal.upsampler = upsampler
    
        return self
    
    def window(self, window: str) -> typing.Self:    
        """
        The time duration
        """
            
        if not len(window) >= 1:
            raise ValueError("len(window) must be >= 1")
        self._internal.window = window
    
        return self
    


class TypeClassicConditions(cogbuilder.Builder[expr.TypeClassicConditions]):
    _internal: expr.TypeClassicConditions

    def __init__(self):
        self._internal = expr.TypeClassicConditions()        
        self._internal.type_val = "classic_conditions"

    def build(self) -> expr.TypeClassicConditions:
        """
        Builds the object.
        """
        return self._internal    
    
    def conditions(self, conditions: list[cogbuilder.Builder[expr.ExprTypeClassicConditionsConditions]]) -> typing.Self:    
        conditions_resources = [r1.build() for r1 in conditions]
        self._internal.conditions = conditions_resources
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        The datasource
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        NOTE: this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def interval_ms(self, interval_ms: float) -> typing.Self:    
        """
        Interval is the suggested duration between time points in a time series query.
        NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
        from the interval required to fill a pixels in the visualization
        """
            
        self._internal.interval_ms = interval_ms
    
        return self
    
    def max_data_points(self, max_data_points: int) -> typing.Self:    
        """
        MaxDataPoints is the maximum number of data points that should be returned from a time series query.
        NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
        from the number of pixels visible in a visualization
        """
            
        self._internal.max_data_points = max_data_points
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        QueryType is an optional identifier for the type of query.
        It can be used to distinguish different types of queries.
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        RefID is the unique identifier of the query, set by the frontend call.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def result_assertions(self, result_assertions: cogbuilder.Builder[expr.ExprTypeClassicConditionsResultAssertions]) -> typing.Self:    
        """
        Optionally define expected query result behavior
        """
            
        result_assertions_resource = result_assertions.build()
        self._internal.result_assertions = result_assertions_resource
    
        return self
    
    def time_range(self, time_range: cogbuilder.Builder[expr.ExprTypeClassicConditionsTimeRange]) -> typing.Self:    
        """
        TimeRange represents the query range
        NOTE: unlike generic /ds/query, we can now send explicit time values in each query
        NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
        """
            
        time_range_resource = time_range.build()
        self._internal.time_range = time_range_resource
    
        return self
    


class TypeThreshold(cogbuilder.Builder[expr.TypeThreshold]):
    _internal: expr.TypeThreshold

    def __init__(self):
        self._internal = expr.TypeThreshold()        
        self._internal.type_val = "threshold"

    def build(self) -> expr.TypeThreshold:
        """
        Builds the object.
        """
        return self._internal    
    
    def conditions(self, conditions: list[cogbuilder.Builder[expr.ExprTypeThresholdConditions]]) -> typing.Self:    
        """
        Threshold Conditions
        """
            
        conditions_resources = [r1.build() for r1 in conditions]
        self._internal.conditions = conditions_resources
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        The datasource
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def expression(self, expression: str) -> typing.Self:    
        """
        Reference to single query result
        """
            
        if not len(expression) >= 1:
            raise ValueError("len(expression) must be >= 1")
        self._internal.expression = expression
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        NOTE: this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def interval_ms(self, interval_ms: float) -> typing.Self:    
        """
        Interval is the suggested duration between time points in a time series query.
        NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
        from the interval required to fill a pixels in the visualization
        """
            
        self._internal.interval_ms = interval_ms
    
        return self
    
    def max_data_points(self, max_data_points: int) -> typing.Self:    
        """
        MaxDataPoints is the maximum number of data points that should be returned from a time series query.
        NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
        from the number of pixels visible in a visualization
        """
            
        self._internal.max_data_points = max_data_points
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        QueryType is an optional identifier for the type of query.
        It can be used to distinguish different types of queries.
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        RefID is the unique identifier of the query, set by the frontend call.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def result_assertions(self, result_assertions: cogbuilder.Builder[expr.ExprTypeThresholdResultAssertions]) -> typing.Self:    
        """
        Optionally define expected query result behavior
        """
            
        result_assertions_resource = result_assertions.build()
        self._internal.result_assertions = result_assertions_resource
    
        return self
    
    def time_range(self, time_range: cogbuilder.Builder[expr.ExprTypeThresholdTimeRange]) -> typing.Self:    
        """
        TimeRange represents the query range
        NOTE: unlike generic /ds/query, we can now send explicit time values in each query
        NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
        """
            
        time_range_resource = time_range.build()
        self._internal.time_range = time_range_resource
    
        return self
    


class TypeSql(cogbuilder.Builder[expr.TypeSql]):
    _internal: expr.TypeSql

    def __init__(self):
        self._internal = expr.TypeSql()        
        self._internal.type_val = "sql"

    def build(self) -> expr.TypeSql:
        """
        Builds the object.
        """
        return self._internal    
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        The datasource
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def expression(self, expression: str) -> typing.Self:    
        if not len(expression) >= 1:
            raise ValueError("len(expression) must be >= 1")
        self._internal.expression = expression
    
        return self
    
    def format(self, format_val: str) -> typing.Self:    
        self._internal.format_val = format_val
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        NOTE: this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def interval_ms(self, interval_ms: float) -> typing.Self:    
        """
        Interval is the suggested duration between time points in a time series query.
        NOTE: the values for intervalMs is not saved in the query model.  It is typically calculated
        from the interval required to fill a pixels in the visualization
        """
            
        self._internal.interval_ms = interval_ms
    
        return self
    
    def max_data_points(self, max_data_points: int) -> typing.Self:    
        """
        MaxDataPoints is the maximum number of data points that should be returned from a time series query.
        NOTE: the values for maxDataPoints is not saved in the query model.  It is typically calculated
        from the number of pixels visible in a visualization
        """
            
        self._internal.max_data_points = max_data_points
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        QueryType is an optional identifier for the type of query.
        It can be used to distinguish different types of queries.
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        RefID is the unique identifier of the query, set by the frontend call.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def result_assertions(self, result_assertions: cogbuilder.Builder[expr.ExprTypeSqlResultAssertions]) -> typing.Self:    
        """
        Optionally define expected query result behavior
        """
            
        result_assertions_resource = result_assertions.build()
        self._internal.result_assertions = result_assertions_resource
    
        return self
    
    def time_range(self, time_range: cogbuilder.Builder[expr.ExprTypeSqlTimeRange]) -> typing.Self:    
        """
        TimeRange represents the query range
        NOTE: unlike generic /ds/query, we can now send explicit time values in each query
        NOTE: the values for timeRange are not saved in a dashboard, they are constructed on the fly
        """
            
        time_range_resource = time_range.build()
        self._internal.time_range = time_range_resource
    
        return self
    


class ExprTypeMathResultAssertions(cogbuilder.Builder[expr.ExprTypeMathResultAssertions]):
    _internal: expr.ExprTypeMathResultAssertions

    def __init__(self):
        self._internal = expr.ExprTypeMathResultAssertions()

    def build(self) -> expr.ExprTypeMathResultAssertions:
        """
        Builds the object.
        """
        return self._internal    
    
    def max_frames(self, max_frames: int) -> typing.Self:    
        """
        Maximum frame count
        """
            
        self._internal.max_frames = max_frames
    
        return self
    
    def type(self, type_val: typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]) -> typing.Self:    
        """
        Type asserts that the frame matches a known type structure.
        Possible enum values:
         - `""` 
         - `"timeseries-wide"` 
         - `"timeseries-long"` 
         - `"timeseries-many"` 
         - `"timeseries-multi"` 
         - `"directory-listing"` 
         - `"table"` 
         - `"numeric-wide"` 
         - `"numeric-multi"` 
         - `"numeric-long"` 
         - `"log-lines"` 
        """
            
        self._internal.type_val = type_val
    
        return self
    
    def type_version(self, type_version: list[int]) -> typing.Self:    
        """
        TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
        contract documentation https://grafana.github.io/dataplane/contract/.
        """
            
        self._internal.type_version = type_version
    
        return self
    


class ExprTypeMathTimeRange(cogbuilder.Builder[expr.ExprTypeMathTimeRange]):
    _internal: expr.ExprTypeMathTimeRange

    def __init__(self):
        self._internal = expr.ExprTypeMathTimeRange()

    def build(self) -> expr.ExprTypeMathTimeRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def from_val(self, from_val: str) -> typing.Self:    
        """
        From is the start time of the query.
        """
            
        self._internal.from_val = from_val
    
        return self
    
    def to(self, to: str) -> typing.Self:    
        """
        To is the end time of the query.
        """
            
        self._internal.to = to
    
        return self
    


class ExprTypeReduceResultAssertions(cogbuilder.Builder[expr.ExprTypeReduceResultAssertions]):
    _internal: expr.ExprTypeReduceResultAssertions

    def __init__(self):
        self._internal = expr.ExprTypeReduceResultAssertions()

    def build(self) -> expr.ExprTypeReduceResultAssertions:
        """
        Builds the object.
        """
        return self._internal    
    
    def max_frames(self, max_frames: int) -> typing.Self:    
        """
        Maximum frame count
        """
            
        self._internal.max_frames = max_frames
    
        return self
    
    def type(self, type_val: typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]) -> typing.Self:    
        """
        Type asserts that the frame matches a known type structure.
        Possible enum values:
         - `""` 
         - `"timeseries-wide"` 
         - `"timeseries-long"` 
         - `"timeseries-many"` 
         - `"timeseries-multi"` 
         - `"directory-listing"` 
         - `"table"` 
         - `"numeric-wide"` 
         - `"numeric-multi"` 
         - `"numeric-long"` 
         - `"log-lines"` 
        """
            
        self._internal.type_val = type_val
    
        return self
    
    def type_version(self, type_version: list[int]) -> typing.Self:    
        """
        TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
        contract documentation https://grafana.github.io/dataplane/contract/.
        """
            
        self._internal.type_version = type_version
    
        return self
    


class ExprTypeReduceSettings(cogbuilder.Builder[expr.ExprTypeReduceSettings]):
    _internal: expr.ExprTypeReduceSettings

    def __init__(self):
        self._internal = expr.ExprTypeReduceSettings()

    def build(self) -> expr.ExprTypeReduceSettings:
        """
        Builds the object.
        """
        return self._internal    
    
    def mode(self, mode: typing.Literal["dropNN", "replaceNN"]) -> typing.Self:    
        """
        Non-number reduce behavior
        Possible enum values:
         - `"dropNN"` Drop non-numbers
         - `"replaceNN"` Replace non-numbers
        """
            
        self._internal.mode = mode
    
        return self
    
    def replace_with_value(self, replace_with_value: float) -> typing.Self:    
        """
        Only valid when mode is replace
        """
            
        self._internal.replace_with_value = replace_with_value
    
        return self
    


class ExprTypeReduceTimeRange(cogbuilder.Builder[expr.ExprTypeReduceTimeRange]):
    _internal: expr.ExprTypeReduceTimeRange

    def __init__(self):
        self._internal = expr.ExprTypeReduceTimeRange()

    def build(self) -> expr.ExprTypeReduceTimeRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def from_val(self, from_val: str) -> typing.Self:    
        """
        From is the start time of the query.
        """
            
        self._internal.from_val = from_val
    
        return self
    
    def to(self, to: str) -> typing.Self:    
        """
        To is the end time of the query.
        """
            
        self._internal.to = to
    
        return self
    


class ExprTypeResampleResultAssertions(cogbuilder.Builder[expr.ExprTypeResampleResultAssertions]):
    _internal: expr.ExprTypeResampleResultAssertions

    def __init__(self):
        self._internal = expr.ExprTypeResampleResultAssertions()

    def build(self) -> expr.ExprTypeResampleResultAssertions:
        """
        Builds the object.
        """
        return self._internal    
    
    def max_frames(self, max_frames: int) -> typing.Self:    
        """
        Maximum frame count
        """
            
        self._internal.max_frames = max_frames
    
        return self
    
    def type(self, type_val: typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]) -> typing.Self:    
        """
        Type asserts that the frame matches a known type structure.
        Possible enum values:
         - `""` 
         - `"timeseries-wide"` 
         - `"timeseries-long"` 
         - `"timeseries-many"` 
         - `"timeseries-multi"` 
         - `"directory-listing"` 
         - `"table"` 
         - `"numeric-wide"` 
         - `"numeric-multi"` 
         - `"numeric-long"` 
         - `"log-lines"` 
        """
            
        self._internal.type_val = type_val
    
        return self
    
    def type_version(self, type_version: list[int]) -> typing.Self:    
        """
        TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
        contract documentation https://grafana.github.io/dataplane/contract/.
        """
            
        self._internal.type_version = type_version
    
        return self
    


class ExprTypeResampleTimeRange(cogbuilder.Builder[expr.ExprTypeResampleTimeRange]):
    _internal: expr.ExprTypeResampleTimeRange

    def __init__(self):
        self._internal = expr.ExprTypeResampleTimeRange()

    def build(self) -> expr.ExprTypeResampleTimeRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def from_val(self, from_val: str) -> typing.Self:    
        """
        From is the start time of the query.
        """
            
        self._internal.from_val = from_val
    
        return self
    
    def to(self, to: str) -> typing.Self:    
        """
        To is the end time of the query.
        """
            
        self._internal.to = to
    
        return self
    


class ExprTypeClassicConditionsConditionsEvaluator(cogbuilder.Builder[expr.ExprTypeClassicConditionsConditionsEvaluator]):
    _internal: expr.ExprTypeClassicConditionsConditionsEvaluator

    def __init__(self):
        self._internal = expr.ExprTypeClassicConditionsConditionsEvaluator()

    def build(self) -> expr.ExprTypeClassicConditionsConditionsEvaluator:
        """
        Builds the object.
        """
        return self._internal    
    
    def params(self, params: list[float]) -> typing.Self:    
        self._internal.params = params
    
        return self
    
    def type(self, type_val: str) -> typing.Self:    
        """
        e.g. "gt"
        """
            
        self._internal.type_val = type_val
    
        return self
    


class ExprTypeClassicConditionsConditionsOperator(cogbuilder.Builder[expr.ExprTypeClassicConditionsConditionsOperator]):
    _internal: expr.ExprTypeClassicConditionsConditionsOperator

    def __init__(self):
        self._internal = expr.ExprTypeClassicConditionsConditionsOperator()

    def build(self) -> expr.ExprTypeClassicConditionsConditionsOperator:
        """
        Builds the object.
        """
        return self._internal    
    
    def type(self, type_val: typing.Literal["and", "or", "logic-or"]) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    


class ExprTypeClassicConditionsConditionsQuery(cogbuilder.Builder[expr.ExprTypeClassicConditionsConditionsQuery]):
    _internal: expr.ExprTypeClassicConditionsConditionsQuery

    def __init__(self):
        self._internal = expr.ExprTypeClassicConditionsConditionsQuery()

    def build(self) -> expr.ExprTypeClassicConditionsConditionsQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def params(self, params: list[str]) -> typing.Self:    
        self._internal.params = params
    
        return self
    


class ExprTypeClassicConditionsConditionsReducer(cogbuilder.Builder[expr.ExprTypeClassicConditionsConditionsReducer]):
    _internal: expr.ExprTypeClassicConditionsConditionsReducer

    def __init__(self):
        self._internal = expr.ExprTypeClassicConditionsConditionsReducer()

    def build(self) -> expr.ExprTypeClassicConditionsConditionsReducer:
        """
        Builds the object.
        """
        return self._internal    
    
    def type(self, type_val: str) -> typing.Self:    
        self._internal.type_val = type_val
    
        return self
    


class ExprTypeClassicConditionsConditions(cogbuilder.Builder[expr.ExprTypeClassicConditionsConditions]):
    _internal: expr.ExprTypeClassicConditionsConditions

    def __init__(self):
        self._internal = expr.ExprTypeClassicConditionsConditions()

    def build(self) -> expr.ExprTypeClassicConditionsConditions:
        """
        Builds the object.
        """
        return self._internal    
    
    def evaluator(self, evaluator: cogbuilder.Builder[expr.ExprTypeClassicConditionsConditionsEvaluator]) -> typing.Self:    
        evaluator_resource = evaluator.build()
        self._internal.evaluator = evaluator_resource
    
        return self
    
    def operator(self, operator: cogbuilder.Builder[expr.ExprTypeClassicConditionsConditionsOperator]) -> typing.Self:    
        operator_resource = operator.build()
        self._internal.operator = operator_resource
    
        return self
    
    def query(self, query: cogbuilder.Builder[expr.ExprTypeClassicConditionsConditionsQuery]) -> typing.Self:    
        query_resource = query.build()
        self._internal.query = query_resource
    
        return self
    
    def reducer(self, reducer: cogbuilder.Builder[expr.ExprTypeClassicConditionsConditionsReducer]) -> typing.Self:    
        reducer_resource = reducer.build()
        self._internal.reducer = reducer_resource
    
        return self
    


class ExprTypeClassicConditionsResultAssertions(cogbuilder.Builder[expr.ExprTypeClassicConditionsResultAssertions]):
    _internal: expr.ExprTypeClassicConditionsResultAssertions

    def __init__(self):
        self._internal = expr.ExprTypeClassicConditionsResultAssertions()

    def build(self) -> expr.ExprTypeClassicConditionsResultAssertions:
        """
        Builds the object.
        """
        return self._internal    
    
    def max_frames(self, max_frames: int) -> typing.Self:    
        """
        Maximum frame count
        """
            
        self._internal.max_frames = max_frames
    
        return self
    
    def type(self, type_val: typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]) -> typing.Self:    
        """
        Type asserts that the frame matches a known type structure.
        Possible enum values:
         - `""` 
         - `"timeseries-wide"` 
         - `"timeseries-long"` 
         - `"timeseries-many"` 
         - `"timeseries-multi"` 
         - `"directory-listing"` 
         - `"table"` 
         - `"numeric-wide"` 
         - `"numeric-multi"` 
         - `"numeric-long"` 
         - `"log-lines"` 
        """
            
        self._internal.type_val = type_val
    
        return self
    
    def type_version(self, type_version: list[int]) -> typing.Self:    
        """
        TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
        contract documentation https://grafana.github.io/dataplane/contract/.
        """
            
        self._internal.type_version = type_version
    
        return self
    


class ExprTypeClassicConditionsTimeRange(cogbuilder.Builder[expr.ExprTypeClassicConditionsTimeRange]):
    _internal: expr.ExprTypeClassicConditionsTimeRange

    def __init__(self):
        self._internal = expr.ExprTypeClassicConditionsTimeRange()

    def build(self) -> expr.ExprTypeClassicConditionsTimeRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def from_val(self, from_val: str) -> typing.Self:    
        """
        From is the start time of the query.
        """
            
        self._internal.from_val = from_val
    
        return self
    
    def to(self, to: str) -> typing.Self:    
        """
        To is the end time of the query.
        """
            
        self._internal.to = to
    
        return self
    


class ExprTypeThresholdConditionsEvaluator(cogbuilder.Builder[expr.ExprTypeThresholdConditionsEvaluator]):
    _internal: expr.ExprTypeThresholdConditionsEvaluator

    def __init__(self):
        self._internal = expr.ExprTypeThresholdConditionsEvaluator()

    def build(self) -> expr.ExprTypeThresholdConditionsEvaluator:
        """
        Builds the object.
        """
        return self._internal    
    
    def params(self, params: list[float]) -> typing.Self:    
        self._internal.params = params
    
        return self
    
    def type(self, type_val: typing.Literal["gt", "lt", "eq", "ne", "gte", "lte", "within_range", "outside_range", "within_range_included", "outside_range_included"]) -> typing.Self:    
        """
        e.g. "gt"
        """
            
        self._internal.type_val = type_val
    
        return self
    


class ExprTypeThresholdConditionsUnloadEvaluator(cogbuilder.Builder[expr.ExprTypeThresholdConditionsUnloadEvaluator]):
    _internal: expr.ExprTypeThresholdConditionsUnloadEvaluator

    def __init__(self):
        self._internal = expr.ExprTypeThresholdConditionsUnloadEvaluator()

    def build(self) -> expr.ExprTypeThresholdConditionsUnloadEvaluator:
        """
        Builds the object.
        """
        return self._internal    
    
    def params(self, params: list[float]) -> typing.Self:    
        self._internal.params = params
    
        return self
    
    def type(self, type_val: typing.Literal["gt", "lt", "eq", "ne", "gte", "lte", "within_range", "outside_range", "within_range_included", "outside_range_included"]) -> typing.Self:    
        """
        e.g. "gt"
        """
            
        self._internal.type_val = type_val
    
        return self
    


class ExprTypeThresholdConditions(cogbuilder.Builder[expr.ExprTypeThresholdConditions]):
    _internal: expr.ExprTypeThresholdConditions

    def __init__(self):
        self._internal = expr.ExprTypeThresholdConditions()

    def build(self) -> expr.ExprTypeThresholdConditions:
        """
        Builds the object.
        """
        return self._internal    
    
    def evaluator(self, evaluator: cogbuilder.Builder[expr.ExprTypeThresholdConditionsEvaluator]) -> typing.Self:    
        evaluator_resource = evaluator.build()
        self._internal.evaluator = evaluator_resource
    
        return self
    
    def loaded_dimensions(self, loaded_dimensions: object) -> typing.Self:    
        self._internal.loaded_dimensions = loaded_dimensions
    
        return self
    
    def unload_evaluator(self, unload_evaluator: cogbuilder.Builder[expr.ExprTypeThresholdConditionsUnloadEvaluator]) -> typing.Self:    
        unload_evaluator_resource = unload_evaluator.build()
        self._internal.unload_evaluator = unload_evaluator_resource
    
        return self
    


class ExprTypeThresholdResultAssertions(cogbuilder.Builder[expr.ExprTypeThresholdResultAssertions]):
    _internal: expr.ExprTypeThresholdResultAssertions

    def __init__(self):
        self._internal = expr.ExprTypeThresholdResultAssertions()

    def build(self) -> expr.ExprTypeThresholdResultAssertions:
        """
        Builds the object.
        """
        return self._internal    
    
    def max_frames(self, max_frames: int) -> typing.Self:    
        """
        Maximum frame count
        """
            
        self._internal.max_frames = max_frames
    
        return self
    
    def type(self, type_val: typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]) -> typing.Self:    
        """
        Type asserts that the frame matches a known type structure.
        Possible enum values:
         - `""` 
         - `"timeseries-wide"` 
         - `"timeseries-long"` 
         - `"timeseries-many"` 
         - `"timeseries-multi"` 
         - `"directory-listing"` 
         - `"table"` 
         - `"numeric-wide"` 
         - `"numeric-multi"` 
         - `"numeric-long"` 
         - `"log-lines"` 
        """
            
        self._internal.type_val = type_val
    
        return self
    
    def type_version(self, type_version: list[int]) -> typing.Self:    
        """
        TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
        contract documentation https://grafana.github.io/dataplane/contract/.
        """
            
        self._internal.type_version = type_version
    
        return self
    


class ExprTypeThresholdTimeRange(cogbuilder.Builder[expr.ExprTypeThresholdTimeRange]):
    _internal: expr.ExprTypeThresholdTimeRange

    def __init__(self):
        self._internal = expr.ExprTypeThresholdTimeRange()

    def build(self) -> expr.ExprTypeThresholdTimeRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def from_val(self, from_val: str) -> typing.Self:    
        """
        From is the start time of the query.
        """
            
        self._internal.from_val = from_val
    
        return self
    
    def to(self, to: str) -> typing.Self:    
        """
        To is the end time of the query.
        """
            
        self._internal.to = to
    
        return self
    


class ExprTypeSqlResultAssertions(cogbuilder.Builder[expr.ExprTypeSqlResultAssertions]):
    _internal: expr.ExprTypeSqlResultAssertions

    def __init__(self):
        self._internal = expr.ExprTypeSqlResultAssertions()

    def build(self) -> expr.ExprTypeSqlResultAssertions:
        """
        Builds the object.
        """
        return self._internal    
    
    def max_frames(self, max_frames: int) -> typing.Self:    
        """
        Maximum frame count
        """
            
        self._internal.max_frames = max_frames
    
        return self
    
    def type(self, type_val: typing.Literal["", "timeseries-wide", "timeseries-long", "timeseries-many", "timeseries-multi", "directory-listing", "table", "numeric-wide", "numeric-multi", "numeric-long", "log-lines"]) -> typing.Self:    
        """
        Type asserts that the frame matches a known type structure.
        Possible enum values:
         - `""` 
         - `"timeseries-wide"` 
         - `"timeseries-long"` 
         - `"timeseries-many"` 
         - `"timeseries-multi"` 
         - `"directory-listing"` 
         - `"table"` 
         - `"numeric-wide"` 
         - `"numeric-multi"` 
         - `"numeric-long"` 
         - `"log-lines"` 
        """
            
        self._internal.type_val = type_val
    
        return self
    
    def type_version(self, type_version: list[int]) -> typing.Self:    
        """
        TypeVersion is the version of the Type property. Versions greater than 0.0 correspond to the dataplane
        contract documentation https://grafana.github.io/dataplane/contract/.
        """
            
        self._internal.type_version = type_version
    
        return self
    


class ExprTypeSqlTimeRange(cogbuilder.Builder[expr.ExprTypeSqlTimeRange]):
    _internal: expr.ExprTypeSqlTimeRange

    def __init__(self):
        self._internal = expr.ExprTypeSqlTimeRange()

    def build(self) -> expr.ExprTypeSqlTimeRange:
        """
        Builds the object.
        """
        return self._internal    
    
    def from_val(self, from_val: str) -> typing.Self:    
        """
        From is the start time of the query.
        """
            
        self._internal.from_val = from_val
    
        return self
    
    def to(self, to: str) -> typing.Self:    
        """
        To is the end time of the query.
        """
            
        self._internal.to = to
    
        return self
    
